@@ -23,7 +23,6 @@ import time
 import warnings
 
 import requests
-import six
 
 import gitlab.config
 from gitlab.const import *  # noqa
@@ -31,7 +30,7 @@ from gitlab.exceptions import *  # noqa
 from gitlab import utils  # noqa
 
 __title__ = "python-gitlab"
-__version__ = "1.14.0"
+__version__ = "2.1.2"
 __author__ = "Gauvain Pocentek"
 __email__ = "gauvainpocentek@gmail.com"
 __license__ = "LGPL3"
@@ -47,8 +46,8 @@ REDIRECT_MSG = (
 
 def _sanitize(value):
     if isinstance(value, dict):
-        return dict((k, _sanitize(v)) for k, v in six.iteritems(value))
-    if isinstance(value, six.string_types):
+        return dict((k, _sanitize(v)) for k, v in value.items())
+    if isinstance(value, str):
         return value.replace("/", "%2F")
     return value
 
@@ -61,8 +60,6 @@ class Gitlab(object):
         private_token (str): The user private token
         oauth_token (str): An oauth token
         job_token (str): A CI job token
-        email (str): The user email or login.
-        password (str): The user password (associated with email).
         ssl_verify (bool|str): Whether SSL certificates should be validated. If
             the value is a string, it is the path to a CA file used for
             certificate validation.
@@ -70,6 +67,8 @@ class Gitlab(object):
         http_username (str): Username for HTTP authentication
         http_password (str): Password for HTTP authentication
         api_version (str): Gitlab API version to use (support for 4 only)
+        pagination (str): Can be set to 'keyset' to use keyset pagination
+        order_by (str): Set order_by globally
     """
 
     def __init__(
@@ -85,12 +84,14 @@ class Gitlab(object):
         api_version="4",
         session=None,
         per_page=None,
+        pagination=None,
+        order_by=None,
     ):
 
         self._api_version = str(api_version)
         self._server_version = self._server_revision = None
-        self._base_url = url
-        self._url = "%s/api/v%s" % (url, api_version)
+        self._base_url = url.rstrip("/")
+        self._url = "%s/api/v%s" % (self._base_url, api_version)
         #: Timeout to use for requests to gitlab server
         self.timeout = timeout
         #: Headers that will be used in request to GitLab
@@ -110,6 +111,8 @@ class Gitlab(object):
         self.session = session or requests.Session()
 
         self.per_page = per_page
+        self.pagination = pagination
+        self.order_by = order_by
 
         objects = importlib.import_module("gitlab.v%s.objects" % self._api_version)
         self._objects = objects
@@ -130,6 +133,7 @@ class Gitlab(object):
         self.projects = objects.ProjectManager(self)
         self.runners = objects.RunnerManager(self)
         self.settings = objects.ApplicationSettingsManager(self)
+        self.appearance = objects.ApplicationAppearanceManager(self)
         self.sidekiq = objects.SidekiqManager(self)
         self.snippets = objects.SnippetManager(self)
         self.users = objects.UserManager(self)
@@ -140,6 +144,7 @@ class Gitlab(object):
         self.features = objects.FeatureManager(self)
         self.pagesdomains = objects.PagesDomainManager(self)
         self.user_activities = objects.UserActivitiesManager(self)
+        self.applications = objects.ApplicationManager(self)
 
     def __enter__(self):
         return self
@@ -200,12 +205,12 @@ class Gitlab(object):
             http_password=config.http_password,
             api_version=config.api_version,
             per_page=config.per_page,
+            pagination=config.pagination,
+            order_by=config.order_by,
         )
 
     def auth(self):
-        """Performs an authentication.
-
-        Uses either the private token, or the email/password pair.
+        """Performs an authentication using private token.
 
         The `user` attribute will hold a `gitlab.objects.CurrentUser` object on
         success.
@@ -491,6 +496,8 @@ class Gitlab(object):
 
         verify = opts.pop("verify")
         timeout = opts.pop("timeout")
+        # If timeout was passed into kwargs, allow it to override the default
+        timeout = kwargs.get("timeout", timeout)
 
         # We need to deal with json vs. data when uploading files
         if files:
@@ -518,6 +525,8 @@ class Gitlab(object):
 
         # obey the rate limit by default
         obey_rate_limit = kwargs.get("obey_rate_limit", True)
+        # do not retry transient errors by default
+        retry_transient_errors = kwargs.get("retry_transient_errors", False)
 
         # set max_retries to 10 by default, disable by setting it to -1
         max_retries = kwargs.get("max_retries", 10)
@@ -531,7 +540,9 @@ class Gitlab(object):
             if 200 <= result.status_code < 300:
                 return result
 
-            if 429 == result.status_code and obey_rate_limit:
+            if (429 == result.status_code and obey_rate_limit) or (
+                result.status_code in [500, 502, 503, 504] and retry_transient_errors
+            ):
                 if max_retries == -1 or cur_retries < max_retries:
                     wait_time = 2 ** cur_retries * 0.1
                     if "Retry-After" in result.headers:
@@ -630,7 +641,7 @@ class Gitlab(object):
         get_all = kwargs.pop("all", False)
         url = self._build_url(path)
 
-        if get_all is True:
+        if get_all is True and as_list is True:
             return list(GitlabList(self, url, query_data, **kwargs))
 
         if "page" in kwargs or as_list is True:
@@ -833,8 +844,10 @@ class GitlabList(object):
             self._current += 1
             return item
         except IndexError:
-            if self._next_url and self._get_next is True:
-                self._query(self._next_url)
-                return self.next()
+            pass
 
-            raise StopIteration
+        if self._next_url and self._get_next is True:
+            self._query(self._next_url)
+            return self.next()
+
+        raise StopIteration
