@@ -120,6 +120,13 @@ class ListMixin(object):
         if self.gitlab.per_page:
             data.setdefault("per_page", self.gitlab.per_page)
 
+        # global keyset pagination
+        if self.gitlab.pagination:
+            data.setdefault("pagination", self.gitlab.pagination)
+
+        if self.gitlab.order_by:
+            data.setdefault("order_by", self.gitlab.order_by)
+
         # We get the attributes that need some special transformation
         types = getattr(self, "_types", {})
         if types:
@@ -163,7 +170,7 @@ class CreateMixin(object):
         return getattr(self, "_create_attrs", (tuple(), tuple()))
 
     @exc.on_http_error(exc.GitlabCreateError)
-    def create(self, data, **kwargs):
+    def create(self, data=None, **kwargs):
         """Create a new object.
 
         Args:
@@ -179,6 +186,9 @@ class CreateMixin(object):
             GitlabAuthenticationError: If authentication is not correct
             GitlabCreateError: If the server cannot perform the request
         """
+        if data is None:
+            data = {}
+
         self._check_missing_create_attrs(data)
         files = {}
 
