@@ -119,6 +119,24 @@ class TestGitlabList(unittest.TestCase):
                 self.assertEqual(l[0]["a"], "b")
                 self.assertEqual(l[1]["c"], "d")
 
+    def test_all_omitted_when_as_list(self):
+        @urlmatch(scheme="http", netloc="localhost", path="/api/v4/tests", method="get")
+        def resp(url, request):
+            headers = {
+                "content-type": "application/json",
+                "X-Page": 2,
+                "X-Next-Page": 2,
+                "X-Per-Page": 1,
+                "X-Total-Pages": 2,
+                "X-Total": 2,
+            }
+            content = '[{"c": "d"}]'
+            return response(200, content, headers, None, 5, request)
+
+        with HTTMock(resp):
+            result = self.gl.http_list("/tests", as_list=False, all=True)
+            self.assertIsInstance(result, GitlabList)
+
 
 class TestGitlabHttpMethods(unittest.TestCase):
     def setUp(self):
@@ -358,6 +376,23 @@ class TestGitlabHttpMethods(unittest.TestCase):
             self.assertRaises(GitlabHttpError, self.gl.http_delete, "/not_there")
 
 
+class TestGitlabStripBaseUrl(unittest.TestCase):
+    def setUp(self):
+        self.gl = Gitlab(
+            "http://localhost/", private_token="private_token", api_version=4
+        )
+
+    def test_strip_base_url(self):
+        self.assertEqual(self.gl.url, "http://localhost")
+
+    def test_strip_api_url(self):
+        self.assertEqual(self.gl.api_url, "http://localhost/api/v4")
+
+    def test_build_url(self):
+        r = self.gl._build_url("/projects")
+        self.assertEqual(r, "http://localhost/api/v4/projects")
+
+
 class TestGitlabAuth(unittest.TestCase):
     def test_invalid_auth_args(self):
         self.assertRaises(
@@ -535,6 +570,62 @@ class TestGitlab(unittest.TestCase):
             self.assertEqual(environment.last_deployment, "sometime")
             self.assertEqual(environment.name, "environment_name")
 
+    def test_project_additional_statistics(self):
+        @urlmatch(
+            scheme="http", netloc="localhost", path="/api/v4/projects/1$", method="get"
+        )
+        def resp_get_project(url, request):
+            headers = {"content-type": "application/json"}
+            content = '{"name": "name", "id": 1}'.encode("utf-8")
+            return response(200, content, headers, None, 5, request)
+
+        @urlmatch(
+            scheme="http",
+            netloc="localhost",
+            path="/api/v4/projects/1/statistics",
+            method="get",
+        )
+        def resp_get_environment(url, request):
+            headers = {"content-type": "application/json"}
+            content = """{"fetches": {"total": 50, "days": [{"count": 10, "date": "2018-01-10"}]}}""".encode(
+                "utf-8"
+            )
+            return response(200, content, headers, None, 5, request)
+
+        with HTTMock(resp_get_project, resp_get_environment):
+            project = self.gl.projects.get(1)
+            statistics = project.additionalstatistics.get()
+            self.assertIsInstance(statistics, ProjectAdditionalStatistics)
+            self.assertEqual(statistics.fetches["total"], 50)
+
+    def test_project_issues_statistics(self):
+        @urlmatch(
+            scheme="http", netloc="localhost", path="/api/v4/projects/1$", method="get"
+        )
+        def resp_get_project(url, request):
+            headers = {"content-type": "application/json"}
+            content = '{"name": "name", "id": 1}'.encode("utf-8")
+            return response(200, content, headers, None, 5, request)
+
+        @urlmatch(
+            scheme="http",
+            netloc="localhost",
+            path="/api/v4/projects/1/issues_statistics",
+            method="get",
+        )
+        def resp_get_environment(url, request):
+            headers = {"content-type": "application/json"}
+            content = """{"statistics": {"counts": {"all": 20, "closed": 5, "opened": 15}}}""".encode(
+                "utf-8"
+            )
+            return response(200, content, headers, None, 5, request)
+
+        with HTTMock(resp_get_project, resp_get_environment):
+            project = self.gl.projects.get(1)
+            statistics = project.issuesstatistics.get()
+            self.assertIsInstance(statistics, ProjectIssuesStatistics)
+            self.assertEqual(statistics.statistics["counts"]["all"], 20)
+
     def test_groups(self):
         @urlmatch(
             scheme="http", netloc="localhost", path="/api/v4/groups/1", method="get"
@@ -584,6 +675,38 @@ class TestGitlab(unittest.TestCase):
             self.assertEqual(user.name, "name")
             self.assertEqual(user.id, 1)
 
+    def test_user_memberships(self):
+        @urlmatch(
+            scheme="http",
+            netloc="localhost",
+            path="/api/v4/users/1/memberships",
+            method="get",
+        )
+        def resp_get_user_memberships(url, request):
+            headers = {"content-type": "application/json"}
+            content = """[
+                          {
+                            "source_id": 1,
+                            "source_name": "Project one",
+                            "source_type": "Project",
+                            "access_level": "20"
+                          },
+                          {
+                            "source_id": 3,
+                            "source_name": "Group three",
+                            "source_type": "Namespace",
+                            "access_level": "20"
+                          }
+                        ]"""
+            content = content.encode("utf-8")
+            return response(200, content, headers, None, 5, request)
+
+        with HTTMock(resp_get_user_memberships):
+            user = self.gl.users.get(1, lazy=True)
+            memberships = user.memberships.list()
+            self.assertIsInstance(memberships[0], UserMembership)
+            self.assertEqual(memberships[0].source_type, "Project")
+
     def test_user_status(self):
         @urlmatch(
             scheme="http",
@@ -720,6 +843,50 @@ class TestGitlab(unittest.TestCase):
             self.gl.users.get(1, lazy=True).activate()
             self.gl.users.get(1, lazy=True).deactivate()
 
+    def test_commit_revert(self):
+        @urlmatch(
+            scheme="http",
+            netloc="localhost",
+            path="/api/v4/projects/1/repository/commits/6b2257ea",
+            method="get",
+        )
+        def resp_get_commit(url, request):
+            headers = {"content-type": "application/json"}
+            content = """{
+            "id": "6b2257eabcec3db1f59dafbd84935e3caea04235",
+            "short_id": "6b2257ea",
+            "title": "Initial commit"
+            }"""
+            content = content.encode("utf-8")
+            return response(200, content, headers, None, 5, request)
+
+        @urlmatch(
+            scheme="http",
+            netloc="localhost",
+            path="/api/v4/projects/1/repository/commits/6b2257ea",
+            method="post",
+        )
+        def resp_revert_commit(url, request):
+            headers = {"content-type": "application/json"}
+            content = """{
+            "id": "8b090c1b79a14f2bd9e8a738f717824ff53aebad",
+            "short_id": "8b090c1b",
+            "title":"Revert \\"Initial commit\\""
+            }"""
+            content = content.encode("utf-8")
+            return response(200, content, headers, None, 5, request)
+
+        with HTTMock(resp_get_commit):
+            project = self.gl.projects.get(1, lazy=True)
+            commit = project.commits.get("6b2257ea")
+            self.assertEqual(commit.short_id, "6b2257ea")
+            self.assertEqual(commit.title, "Initial commit")
+
+        with HTTMock(resp_revert_commit):
+            revert_commit = commit.revert(branch="master")
+            self.assertEqual(revert_commit["short_id"], "8b090c1b")
+            self.assertEqual(revert_commit["title"], 'Revert "Initial commit"')
+
     def test_update_submodule(self):
         @urlmatch(
             scheme="http", netloc="localhost", path="/api/v4/projects/1$", method="get"
@@ -770,6 +937,60 @@ class TestGitlab(unittest.TestCase):
             self.assertEqual(ret["message"], "Message")
             self.assertEqual(ret["id"], "ed899a2f4b50b4370feeea94676502b42383c746")
 
+    def test_import_github(self):
+        @urlmatch(
+            scheme="http",
+            netloc="localhost",
+            path="/api/v4/import/github",
+            method="post",
+        )
+        def resp_import_github(url, request):
+            headers = {"content-type": "application/json"}
+            content = """{
+            "id": 27,
+            "name": "my-repo",
+            "full_path": "/root/my-repo",
+            "full_name": "Administrator / my-repo"
+            }"""
+            content = content.encode("utf-8")
+            return response(200, content, headers, None, 25, request)
+
+        with HTTMock(resp_import_github):
+            base_path = "/root"
+            name = "my-repo"
+            ret = self.gl.projects.import_github("githubkey", 1234, base_path, name)
+            self.assertIsInstance(ret, dict)
+            self.assertEqual(ret["name"], name)
+            self.assertEqual(ret["full_path"], "/".join((base_path, name)))
+            self.assertTrue(ret["full_name"].endswith(name))
+
+    def test_applications(self):
+        content = '{"name": "test_app", "redirect_uri": "http://localhost:8080", "scopes": ["api", "email"]}'
+        json_content = json.loads(content)
+
+        @urlmatch(
+            scheme="http",
+            netloc="localhost",
+            path="/api/v4/applications",
+            method="post",
+        )
+        def resp_application_create(url, request):
+            headers = {"content-type": "application/json"}
+            return response(200, json_content, headers, None, 5, request)
+
+        with HTTMock(resp_application_create):
+            application = self.gl.applications.create(
+                {
+                    "name": "test_app",
+                    "redirect_uri": "http://localhost:8080",
+                    "scopes": ["api", "email"],
+                    "confidential": False,
+                }
+            )
+            self.assertEqual(application.name, "test_app")
+            self.assertEqual(application.redirect_uri, "http://localhost:8080")
+            self.assertEqual(application.scopes, ["api", "email"])
+
     def _default_config(self):
         fd, temp_path = tempfile.mkstemp()
         os.write(fd, valid_config)
